%% MakeFigure_forCFR.m
%  THIS SCRIPT REPLICATES FIGURE 1 IN N. BORRI AND G. RAGUSA "Sensitivity, Moment Conditions, and the Risk-free Rate in Yogo (2006)"
%  NOTE: data are from M. Yogo "A Consumption-Based Explanation of Expected Stock Returns" and available on Yogo's website
%
%
%  1 ) first plot descriptive statistics of data to make sure we have same variables as in Yogo
%  2) plot figure 1 in our paper (SDF in YOGO vs.  SDF IMPLIED BY EULER EQUATION WITH RESPECT TO RISK-FREE RATE

clear;
close all;
clc;

%%

addpath(strcat(pwd,'/data/'));

% (1) LOAD DATA FROM YOGO (2006)

% Details on data:
% - FF_Q: cols -> year, quarter, 25 portfolios sorted on size and BM
% - VWindust: cols -> year, quarter, 25 portfolios size and BM by industry
% - VWbeta: cols -> year, quarter, 25 portfolios sorted on HML betas
% - factors: cols -> year, quarter, Mkt, SMB, HML (note: Mkt is excess)
% - Returns: cols -> year, quarter, 10 cols
% - Quarterly: cols-> year, quarter, 13 cols last is 1/0

load Quarterly.mat;
load Returns.mat; 
load FF_Q.mat;                      % FF 5x5 portfolios (quarterly)
load VWindust.mat;                  % FF 5x5 industry
load VWbeta.mat;                    % FF 5x5 beta
load factors.mat;                   % FF 3 factors (quarterly)

% CREATE VARIABLES

non      = Quarterly(:,4);          % non-durables growth rate
dur      = Quarterly(:,5);          % durables growth rate
durnon   = Quarterly(:,10);         % durables stock relative to nondurable
Ldurnon  = Quarterly(:,11);
Rprice   = Quarterly(:,12)./Quarterly(:,13);
Lprice   = Quarterly(:,13);         % relative price of durables

Mkt      = factors(:,3);            % FF Mkt
SMB      = factors(:,4);            % FF SMB
HML      = factors(:,5);            % FF HML

RW       = (1+Returns(:,4))./Quarterly(:,14);

GrossRet = (1+Returns(:,3))./Quarterly(:,14);

dateq    = NaN(size(Returns(:,1))); % quarterly freq. date
for t = 1:length(dateq);
    dateq(t) = datenum(Returns(t,1),Returns(t,2),eomday(Returns(t,1),Returns(t,2)));
end;

% FF real:

Mkt_real = Mkt./Quarterly(:,14);
SMB_real = SMB./Quarterly(:,14);
HML_real = HML./Quarterly(:,14);
FF25_real= (FF_Q(:,3:27)-repmat(Returns(:,3),1,25))./repmat(Quarterly(:,14),1,25);  % excess returns (real)
FF25     = (FF_Q(:,3:27)-repmat(Returns(:,3),1,25));                                % excess returns (nominal)

% PLOTS

% NBER recession dates

dates_NBER   = [datenum(1953,6,eomday(1953,6)) datenum(1954,6,eomday(1954,6)); ...
    datenum(1957,9,eomday(1957,9)) datenum(1958,6,eomday(1958,6));...
    datenum(1960,6,eomday(1960,6)) datenum(1961,3,eomday(1961,3));...
    datenum(1969,12,eomday(1969,12)) datenum(1970,12,eomday(1970,12));...
    datenum(1973,12,eomday(1973,12)) datenum(1975,3,eomday(1975,3));...
    datenum(1980,3,eomday(1980,3)) datenum(1980,9,eomday(1980,9));...
    datenum(1981,9,eomday(1981,9)) datenum(1982,12,eomday(1982,12));...
    datenum(1990,9,eomday(1990,9)) datenum(1991,3,eomday(1991,3))];
            
% Build a step function

stepfunctionNBER = zeros(size(dateq,1),1);
for j=1:size(dates_NBER,1)
    line_start_crisis = find(dateq(:,1)==dates_NBER(j,1));
    line_end_crisis   = find(dateq(:,1)==dates_NBER(j,2));
    stepfunctionNBER(line_start_crisis:line_end_crisis,1) = 1;
end;



% THIS is Figure 1 in Yogo (2006), page 546 of original paper

figure(1);
plot(dateq,durnon,'LineWidth',2,'Color','Black');
hold on;
plot(dateq,Lprice,'--r');
y_min = 1; y_max = 2.5;
eps1 = 0.105;
lightgray  = [238/255 233/255 233/255];
bar1 = bar(dateq(:,1),stepfunctionNBER*(y_max-eps1),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
bar1 = bar(dateq(:,1),stepfunctionNBER*(y_min+eps1),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
plot(dateq,durnon,'LineWidth',2,'Color','Black');
plot(dateq,Lprice,'--r');
hold off;
datetick;
xlim([dateq(1) dateq(end)]);
ylim([1 2.4]);
leg =legend('Durable stock relative to non-durable consumption (D/C)','Relative price of durables','Location','Best');
set(leg,'FontSize',8);legend('boxoff');
filename = 'Figure_Yogo1';
print('-depsc2',filename); 

% This is Figure 2 in Yogo (2006), page 548 of orignal paper

figure(2);
subplot(2,1,1);
plot(dateq,100*non,'--r');
hold on;
plot(dateq,100*dur,'LineWidth',2,'Color','Blue');
y_min = -1.5; y_max = 2.5;
eps = 0.07;
lightgray  = [238/255 233/255 233/255];
bar1 = bar(dateq(:,1),stepfunctionNBER*(y_max-eps),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
bar2 = bar(dateq,stepfunctionNBER*(y_min+eps),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
line([dateq(1) dateq(end)],[0 0],'Color','Black');
plot(dateq,100*non,'--r');
plot(dateq,100*dur,'LineWidth',2,'Color','Blue');
hold off;
datetick;
xlim([dateq(1) dateq(end)]);
ylim([-1.5 2.5]);
ylabel('% per quarter');
leg =legend('Nondurables','Durables','Location','Best');
set(leg,'FontSize',7);legend('boxoff');
title('Nondurables Consumption and Durables Stock Growth');
subplot(2,1,2);
plot(dateq,100*(non-dur),'LineWidth',2,'Color','Green');
hold on;
y_min = -3; y_max = 2.0;
eps = 0.07;
lightgray  = [238/255 233/255 233/255];
bar1 = bar(dateq(:,1),stepfunctionNBER*(y_max-eps),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
bar2 = bar(dateq,stepfunctionNBER*(y_min+eps),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
line([dateq(1) dateq(end)],[0 0],'Color','Black');
line([dateq(1) dateq(end)],[0 0],'Color','Black');
plot(dateq,100*(non-dur),'LineWidth',2,'Color','Green');
hold off;
datetick;
xlim([dateq(1) dateq(end)]);
ylim([-3 2.0]);
ylabel('% per quarter');
title('Nondurables Consumption Growth Minus Durables Stock Growth');
filename = 'Figure_Yogo2';
print('-depsc2',filename);

% TABLE I YOGO: Descriptive statistics
% In Yogo file is summ.prg
% NOTE: Yogo use full sample and consider FF factors not discounted by
% price level

fact = [Mkt SMB HML non dur];

col_mean = mean(fact,1);
col_std  = std(fact,1); % NB: Yogo compute variance dividing by n and not n-1. I do the same
col_ac   = [corr(fact(2:end,1),fact(1:end-1,1));corr(fact(2:end,2),fact(1:end-1,2));corr(fact(2:end,3),fact(1:end-1,3));corr(fact(2:end,4),fact(1:end-1,4));corr(fact(2:end,5),fact(1:end-1,5))];
cor_mat  = corr(fact);

disp(' ');
disp('Descriptive statistics YOGO (2006)');
fprintf('%6s \n','\hline\hline');
fprintf('%6s \n','& & & & \multicolumn{4}{c}{Correlation} \\');
fprintf('%6s \n','\cline{5-8}');
fprintf('%6s \n','Variable & Mean (\%) & SD (\%) & AC & Market & SMB & HML & ND \\');
fprintf('%6s \n','\hline');
fprintf(' %6s %6.3f %6s %6.3f %6s %6.3f %6s \n','Market & ',100*col_mean(1),'&',100*col_std(1),'&',col_ac(1),' \\');
fprintf(' %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s \n','SMB & ',100*col_mean(2),'&',100*col_std(2),'&',col_ac(2),'&',cor_mat(2,1),' \\');
fprintf(' %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s \n','HML & ',100*col_mean(3),'&',100*col_std(3),'&',col_ac(3),'&',cor_mat(3,1),'&',cor_mat(3,2),' \\');
fprintf(' %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s \n','Nondurables & ',100*col_mean(4),'&',100*col_std(4),'&',col_ac(4),'&',cor_mat(4,1),'&',cor_mat(4,2),'&',cor_mat(4,3),' \\');
fprintf(' %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f %6s %6.3f  %6s %6.3f %6s \n','Durables & ',100*col_mean(5),'&',100*col_std(5),'&',col_ac(5),'&',cor_mat(5,1),'&',cor_mat(5,2),'&',cor_mat(5,3),'&',cor_mat(5,4),' \\');
fprintf('%6s \n','\hline\hline');


%% FIGURE: SIMULATED SDF (use equation 8 page 552)
%  NOTE: use estimates of non-linear model (Table II Yogo page 552)

% Define function corresponding to equation 9 in Yogo (i.e, v(D/C)):
% x = D/C
% y = \alpha
% z = \rho

vfun = @(x,y,z) (1-y+y*x.^(1-1/z)).^(1/(1-1/z));

% Yogo non-linear estimates from Table II

[num_yogo,txt_yogo] = xlsread('tables_yogo.xlsx','YogoTable2');


%% FIGURE: SDF YOGO ALL PORTFOLIOS vs. RISK-FREE

betayogo = num_yogo(5,4);
alphayogo= num_yogo(4,4);
rhoyogo  = num_yogo(3,4);
gammayogo= num_yogo(2,4);
sigmayogo= num_yogo(1,4);
kappayogo= (1-gammayogo)/(1-1/sigmayogo);

% Simulate SDF using equation 8 in Yogo

SDF = (betayogo*(1+non(2:end)).^(-1/sigmayogo).*(vfun(durnon(2:end),alphayogo,rhoyogo)./vfun(durnon(1:end-1),alphayogo,rhoyogo)).^(1/rhoyogo-1/sigmayogo).*RW(2:end).^(1-1/kappayogo)).^kappayogo;

figure('Name','SDF');
plot(dateq(2:end),SDF,'LineWidth',1,'Color','Black');
hold on;
plot(dateq(2:end),1./(1+Returns(2:end,3)),'--b','LineWidth',1);
y_min = 0; y_max = 1.6;
eps = 0.01;
lightgray  = [238/255 233/255 233/255];
bar1 = bar(dateq(:,1),stepfunctionNBER*(y_max-eps),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
bar2 = bar(dateq,stepfunctionNBER*(y_min+eps),'FaceColor',lightgray,'EdgeColor',lightgray,'BarWidth',1);
plot(dateq(2:end),SDF,'LineWidth',2,'Color','Black');
plot(dateq(2:end),1./(1+Returns(2:end,3)),'--b','LineWidth',2);
hold off;
datetick;
leg = legend('Durable (Yogo)','1/R^f','Location','north');set(leg,'Fontsize',8);legend('boxoff');
xlim([dateq(2) dateq(end)]);
ylim([0 1.6]);
%title('SDF (Estimates in Yogo (2006), all portfolios)');
filename = 'Figure_YogoSDF_single';
print('-depsc2',filename);