################################################################################
## Consumption Capital Asset Pricing Model
## GMM Estimation
## Last modified: Th Sep 29 23:38:35 2016
##
## Authors: Nicola Borri and Giuseppe Ragusa
################################################################################

rm(list = ls())


################################################################################
# RANDOM SEED: 
################################################################################
set.seed(1234567)
################################################################################

################################################################################
# Load packages:
################################################################################
if !(require(pacman)) install.packages("pacman")
p_load(rgenoud, MASS, texreg,  install = TRUE)

""

################################################################################
# Genetic Algorithm settings
################################################################################

pop.size <- 20000
gradient.check <- FALSE
solution.tolerance <- 0.0001





################################################################################
# Code
################################################################################

source("moment_function.R")
FF_Q      <-  read.table("data/txt/FF_Q.txt", header = FALSE)
Quarterly <-  read.table("data/txt/Quarterly.txt", header = FALSE)
Factors_Q <-  read.table("data/txt/factors.txt", header = FALSE)
VWindust  <-  read.table("data/txt/VWindust.txt", header = FALSE)
VWbeta    <-  read.table("data/txt/VWbeta.txt", header = FALSE)
Returns   <-  read.table("data/txt/tbill.txt", header = FALSE)


getdata <- function(portfolio = c("famafrench",
                                  "industry",
                                  "beta",
                                  "all",
                                  "conditional")) {
    n <- nrow(Quarterly)
    if (portfolio == "conditional")
      idx <- (3:n)
    else
      idx <- (1:n)

    non      <- Quarterly[idx, 4]
    durnon   <- Quarterly[idx, 10]
    Ldurnon  <- Quarterly[idx, 11]
    Rprice   <- Quarterly[idx, 12] / Quarterly[idx, 13]
    Lprice   <- Quarterly[idx, 13]
    RW       <- (1 + Returns[idx, 4]) / Quarterly[idx, 14]
    GrossRet <- (1 + Returns[idx, 3]) / Quarterly[idx, 14]

    switch(portfolio,
      famafrench = {
        ExcessRet = apply(FF_Q[, 3:27], 2, function(u)
          (u - Returns[, 3]) / Quarterly[, 14])
        instrum   = array(1, n)
      },
      industry = {
        ExcessRet = apply(VWindust[, 3:26], 2, function(u)
          (u - Returns[, 3]) / Quarterly[, 14])
        instrum   = array(1, n)
      },
      beta = {
        ExcessRet = apply(VWbeta[, 3:27], 2, function(u)
          (u - Returns[, 3]) / Quarterly[, 14])
        instrum   = array(1, n)
      },
      all = {
        ExcessRet <- cbind(
          apply(FF_Q[, 3:27], 2, function(u)
            (u - Returns[, 3]) / Quarterly[, 14]),
          apply(VWindust[, 3:26], 2, function(u)
            (u - Returns[, 3]) / Quarterly[, 14]),
          apply(VWbeta[, 3:27], 2, function(u)
            (u - Returns[, 3]) / Quarterly[, 14])
        )
        instrum   = array(1, n)
      },
      conditional = {
        ExcessRet = FF_Q[idx, 3:5] / Quarterly[idx, 14]
        instrum   = cbind(array(1, n - 2),
                          Quarterly[1:(n - 2), 4:5],
                          Returns[1:(n - 2), 9:12])
      }
    )

    Nexc   = ncol(ExcessRet)
    Nport  = NCOL(GrossRet) + Nexc
    Ninst  = NCOL(instrum)
    Nmom   = (Nport + 1) * Ninst
    Nobs   = nrow(ExcessRet)

    list(
      non = non,
      durnon = durnon,
      Ldurnon = Ldurnon,
      Rprice = Rprice,
      Lprice = Lprice,
      RW = RW,
      GrossRet = GrossRet,
      ExcessRet = as.matrix(ExcessRet),
      instrum = as.matrix(instrum),
      Nexc = Nexc,
      Nobs = Nobs,
      Nmom = Nmom,
      Ninst = Ninst
    )
  }

pastedata <- function(data) {
  for (j in names(data)) {
    assign(j, data[[j]], envir = .GlobalEnv)
  }
}

obj <- function(theta, W) {
  g <- momf_(theta)
  gn <- colSums(g)
  f <- gn %*% W %*% gn
  ifelse(is.finite(f), f, .Machine$double.xmax^(1/46))
}

dobj <- function(theta, W) {
  g <- momf_(theta)
  gn <- colSums(g)
  dg <- dmomf_(theta)
  2 * gn %*% W %*% dg
}

varhac <- function(dat, imax, ilag, imodel, iprint) {
  ## VARHAC ESTIMATOR FOR THE COVARIANCE MATRIX (SPECTRAL DENSITY AT FREQUENCY ZERO)

  ## INPUT:

  ## dat:        input matrix where each row corresponds to a different observation
  ## imax:       maximum lag order considered
  ##             0 = no correction for serial correlation will be made
  ## ilag:       1 = all elements enter with the same lag
  ##             2 = own element can enter with a different lag
  ##             3 = only the own lag enters
  ## imodel:     1 = AIC is used
  ##             2 = BIC is used
  ##             3 = a fixed lag order equal to imax is used
  ## iprint:     1 = the chosen lag length will be printed on the screen

  ## OUTPUT:

  ## ccc:        VARHAC estimator

  nt   <- nrow(dat);
  kdim <- ncol(dat);

  ddd <- dat[(imax + 1):nt, ]
  minres <- ddd
  aic <- log(colSums(minres ^ 2))

  if (ilag == 2) {
    minorder <- matrix(0,kdim,2)
  } else {
    minorder <- matrix(0, kdim,1)
  }

  if (imax > 0) {
    minpar <- matrix(0, kdim,kdim*imax);
    if (ilag == 1) {
      for (k in 1:kdim) {
        dep <- ddd[,k]
        for (iorder in 1:imax) {
          if (iorder == 1) {
            ex = dat[imax:(nt - 1), ]
          } else {
            ex = cbind(ex, dat[(imax + 1 - iorder):(nt - iorder), ])
          }

          if (imodel != 3) {
            ## Run the VAR ##
            b <- solve(crossprod(ex), crossprod(ex, dep))
            resid <- dep - ex %*% b
            ## Compute the model selection criterion
            npar <- kdim * iorder

            if (imodel == 1) {
              aicnew <- log(colSums(resid ^ 2)) + 2 * npar / (nt - imax)
            } else {
              aicnew <- log(colSums(resid ^ 2)) + log(nt - imax) * npar / (nt - imax)
            }

            if (aicnew < aic[k]) {
              aic[k] = aicnew
              minpar[k, 1:(kdim * iorder)] = t(b)
              minres[, k] <- resid
              minorder[k] <- iorder
            }
          }
        }

        if (imodel == 3) {
          b <- solve(crossprod(ex), crossprod(ex, dep))
          resid <- dep - ex %*% b
          minpar[k,] <- t(b)
          minres[,k] <- resid;
          minorder[k] <- imax;
        }
      }
    }

    if (ilag == 3) {
      for (k in 1:kdim) {
        dep <- ddd[,k]

        for (iorder in 1:imax) {
          if (iorder == 1)
            ex = dat[imax:(nt - 1), k]
          else
            ex = cbind(ex, dat[(imax + 1 - iorder):(nt - iorder), k])


          if (imodel != 3) {
            ## Run the VAR ##
            b <- solve(crossprod(ex), crossprod(ex, dep))
            resid <- dep - ex %*% b
            ## Compute the model selection criterion ##
            npar <- iorder
            if (imodel == 1)
              aicnew <- log(colSums(resid ^ 2)) + 2 * npar / (nt - imax)
            else
              aicnew <- log(colSums(resid ^ 2)) + log(nt - imax) * npar / (nt - imax)


            if (aicnew < aic[k]) {
              aic[k] <- aicnew

              for (i in 1:iorder) {
                minpar[k, k + (i - 1) * kdim] = b[i]
              }

              minres[,k] <- resid
              minorder[k] <- iorder
            }
          }
        }

        if (imodel == 3) {
          b <- solve(crossprod(ex), crossprod(ex, dep))
          resid <- dep - ex %*% b
          for (i in 1:imax) {
            minpar[k, k + (i - 1) * kdim] <- b[i]
          }

          minres[,k] <- resid
          minorder[k] <- imax
        }
      }
    }

    if (ilag == 2) {
      for (k in 1:kdim) {
        dep = ddd[,k]

        if (k == 1)
          dat2 <- dat[, 2:kdim]
        else if (k == kdim)
          dat2 <- dat[, 1:(kdim - 1)]
        else
          dat2 <- cbind(dat[, 1:(k - 1)], dat[, (k + 1):kdim])


        for (iorder in 0:imax) {
          if (iorder == 1) {
            ex1 <- dat[imax:(nt - 1), k]
          } else if (iorder > 1) {
            ex1 <- cbind(ex1, dat[(imax + 1 - iorder):(nt - iorder), k])
          }

          for (iorder2 in 0:imax) {
            if (iorder2 == 1) {
              ex2 <- dat2[imax:(nt - 1), ]
            } else if (iorder2 > 1) {
              ex2 <- cbind(ex2, dat2[(imax + 1 - iorder2):(nt - iorder2), ])
            }


            if (iorder + iorder2 == 0) {
              break
            } else if (iorder == 0) {
              ex <- ex2
            }
            else if (iorder2 == 0) {
              ex = ex1
            } else {
              ex <- cbind(ex1, ex2)
            }

            ## Run the VAR ##
            b <- solve(crossprod(ex), crossprod(ex, dep))
            resid <- dep - ex %*% b

            ## Compute the model selection criterion ##

            npar <- iorder + iorder2 * (kdim - 1)

            if (imodel == 1) {
              aicnew <- log(colSums(resid ^ 2)) + 2 * npar / (nt - imax)
            } else {
              aicnew <- log(colSums(resid ^ 2)) + log(nt - imax) * npar / (nt - imax)
            }


            if (aicnew < aic[k]) {
              aic[k] <- aicnew
              minpar[k,] <- matrix(0, 1,kdim*imax)

              for (i in 1:iorder) {
                minpar[k, k + (i - 1) * kdim] <- b[i]
              }

              if (iorder2 > 0) {
                for (i in 1:iorder2) {
                  if (k==1) {
                    minpar[k, (2 + (i - 1) * kdim):(kdim + (i - 1) * kdim)] <-
                      b[(iorder + 1 + (i - 1) * (kdim - 1)):(iorder + i * (kdim - 1))]
                  } else if (k == kdim)
                    minpar[k, (1 + (i - 1) * kdim):(kdim - 1 + (i - 1) * kdim)] <-
                      b[(iorder + 1 + (i - 1) * (kdim - 1)):(iorder + i * (kdim - 1))]
                  else {
                    minpar[k, (1 + (i - 1) * kdim):(k - 1 + (i - 1) * kdim)] <-
                      b[(iorder + 1 + (i - 1) * (kdim - 1)):(iorder + k - 1 + (i - 1) * (kdim -
                                                                                           1))]
                    minpar[k, (k + 1 + (i - 1) * kdim):(kdim + (i - 1) * kdim)] <-
                      b[(iorder + k + (i - 1) * (kdim - 1)):(iorder + i * (kdim - 1))]
                  }
                }
              }

              minres[,k] <- resid;
              minorder[k,] <- cbind(iorder, iorder2)
            }
          }
        }
      }
    }
  }

  ## Changed here from original.
  ## deflating covariance by n-k, not n-max(lag)
  ## cov <- t(minres)%*%minres/(nt-imax)
  cov <- t(minres) %*% minres / c(nt - imax)

  bbb <- diag(1, kdim)

  if (imax > 0) {
    for (iorder in 1:imax)
      bbb <- bbb - minpar[, ((iorder - 1) * kdim + 1):(iorder * kdim)]
  }

  ccc <- solve(bbb) %*% cov %*% solve(t(bbb))

  ## PRINT LAG LENGTH IF (IPRINT = 1) ##

  if (iprint == 1 & imax > 0) {
    if (imodel == 1) {
      cat("LAG ORDERS CHOSEN WITH AIC: ")
    } else if (imodel == 2)
      cat("LAG ORDERS CHOSEN WITH BIC: ")
    else
      cat("FIXED LAG ORDERS")
    cat(minorder, '\n')
  }
  ## RETURN THE VARHAC ESTIMATOR ##
  return(ccc)
}



## Initial value as in yogo

## Portfolio 1

theta_1 <- c(0.0018090568, 189.12650, 0.78954919, 0.90711062, 0.92)


## This is how the variance covariance matrix is calculated
## Omega = varhac(r,lag,2,1,1);

Domains <-  matrix(c(0.00001,    0.00001,   0.00001,  0.0001, 0.001,
                     2,              300,         2,       1, 1), 5, 2)

results <- list()

for (dt in c("famafrench", "industry", "beta", "all", "conditional")) {
  data <- getdata(dt)
  pastedata(data)
  deprec <- 0.94
  ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ## First step
  ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  W <- diag(Nmom)
  fs <- genoud(
    obj,
    5,
    pop.size = pop.size,
    max.generation = 100,
    hard.generation.limit = TRUE,
    solution.tolerance = solution.tolerance,
    starting.values = theta_1,
    Domains = Domains,
    gr = NULL,
    W = W,
    default.domains = NULL,
    BFGS = T,
    boundary.enforcement = 2,
    gradient.check = gradient.check,
    unif.seed = 787573,
    int.seed = 787573
  )

  ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ## Calculate Fisrt Step SE, adj-Jstat
  ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


  g <- momf_(fs$par)
  n_obs <- nrow(g)

  G <- dmomf_(fs$par)/n_obs
  gn <- colSums(g)/n_obs
  S <- varhac(dat = g, 1, 2, 1, 1)


  ## FIRST STEP - ADJUSTED J-STAT
  A    <- W - G%*%solve(t(G)%*%W%*%G)%*%t(G)%*%W
  Jadj <- n_obs*gn%*%ginv(A%*%S%*%t(A))%*%gn

  B <- solve(t(G) %*% W %*% G)
  M <- t(G) %*% W %*% S %*% W %*% G

  V_fs <- B %*% M %*% B / n_obs
  se_fs <- sqrt(diag(V_fs))/sqrt(n_obs)

  θ <- fs$par
  D = c(1,  -1)
  sep_fs = (θ[1] - θ[3]) ^ 2. / (D %*% V_fs[c(1, 3), c(1, 3)] %*% D)
  pv_sep_fs = pchisq(sep_fs, df = 1, lower = FALSE)
  # Test for time separability
  D <-  c(1,  θ[2] ^ (-2))
  timesep_fs <-  (θ[1] - 1 / θ[2]) ^ 2 / (D %*% V_fs[1:2, 1:2] %*% D)
  pv_timesep_fs = pchisq(timesep_fs, df = 1, lower = FALSE)

  ## Second Step
  W <- solve(varhac(dat = momf_(fs$par), 1, 2, 1, 1))/n_obs
  ss <-
    genoud(
      obj,
      5,
      pop.size = pop.size,
      starting.values = fs$par,
      max.generation = 100,
      hard.generation.limit = TRUE,
      solution.tolerance=solution.tolerance,
      Domains = Domains,
      gr = NULL,
      W = W,
      default.domains = NULL,
      BFGS = T,
      boundary.enforcement = 2,
      gradient.check = gradient.check,
      unif.seed = 7875,
      int.seed = 7875
    )

  G <- dmomf_(ss$par)
  g <- momf_(ss$par)
  gn <- colSums(g)/n_obs
  S <- varhac(dat = g, 1, 2, 1, 1)
  J_ss <- gn %*% solve(S) %*% gn * n_obs
  V_ss <- solve(t(G) %*% solve(S) %*% G)/n_obs
  se_ss <- sqrt(diag(V_ss))

  θ <- ss$par
  D = c(1,  -1)
  sep_ss = (θ[1] - θ[3]) ^ 2. / (D %*% V_ss[c(1, 3), c(1, 3)] %*% D)
  pv_sep_ss = pchisq(sep_ss, df = 1, lower = FALSE)
  # Test for time separability
  D <-  c(1,  θ[2] ^ (-2))
  timesep_ss <-  (θ[1] - 1 / θ[2]) ^ 2 / (D %*% V_ss[1:2, 1:2] %*% D)
  pv_timesep_ss = pchisq(timesep_ss, df = 1, lower = FALSE)

  results[[dt]] <- list()
  results[[dt]][["firststage"]] <- list(
    coef = fs$par,
    se = se_fs,
    sep = sep_fs,
    pv_sep = pv_sep_fs,
    timesep = timesep_fs,
    pv_timesep = pv_timesep_fs,
    J = Jadj,
    Jadj = Jadj,
    W = diag(Nmom)
  )

  results[[dt]][["secondstage"]] <- list(
    coef = ss$par,
    se = se_ss,
    sep = sep_ss,
    pv_sep = pv_sep_ss,
    timesep = timesep_ss,
    pv_timesep = pv_timesep_ss,
    J = J_ss,
    Jadj = J_ss,
    W = W
  )
}


# First Stage Results
fakedata <- data.frame(matrix(rnorm(900), 100, 9))
fm <- lm(X1 ~ X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9 - 1, data = fakedata)
stage <- "firststage"

cl <- c("Fama-French", "Industry & BE/ME", "Beta-sorted", "All portfolios", "FF3")

coef <- se <- pval <- list()
for (dt in c("famafrench", "industry", "beta", "all", "conditional")) {
  df <- results[[dt]][["firststage"]]
  coef[[dt]] <- df$coef
  se[[dt]] <- df$se
  
  ## Add separability test
  coef[[dt]] <- c(coef[[dt]], df$sep, df$timesep)
  se[[dt]] <- c(se[[dt]], df$pv_sep, df$pv_timesep)
  pval[[dt]] <-  2*(1-pnorm(abs(df$coef/df$se)))
  ## Add J statistics
  coef[[dt]] <- c(coef[[dt]], df$Jadj)
  se[[dt]] <- c(se[[dt]], pchisq(df$Jadj, df = nrow(df$W) - 5, lower = FALSE))
}


screenreg(
  list(fm, fm, fm, fm, fm), file = "firststage.txt",
  stars = numeric(0),
  override.coef = coef,
  override.se = se,
  custom.coef.names = c(
    "$\\sigma$",
    "$\\gamma$",
    "$\\rho$",
    "$\\alpha$",
    "$\\beta$",
    "Test for $\\sigma = \\rho$",
    "Test for $\\sigma=1/\\gamma$",
    "$J-stat$"
  ),
  custom.model.names = cl,
  groups = list(
    "Parameters" = 1:5,
    "Tests of separability" = 6:7,
    "Specification test" = 8:8
  ),
  include.rsquared = F,
  include.adjrs = FALSE,
  include.nobs = F,
  include.rmse = FALSE
)


texreg(
  list(fm, fm, fm, fm, fm), file = "firststage.tex",
  override.coef = coef,
  override.se = se,
  stars = numeric(0),
  custom.coef.names = c(
    "$\\sigma$",
    "$\\gamma$",
    "$\\rho$",
    "$\\alpha$",
    "$\\beta$",
    "Test for $\\sigma = \\rho$",
    "Test for $\\sigma=1/\\gamma$",
    "$J-stat$"
  ),
  custom.model.names = cl,
  groups = list(
    "Parameters" = 1:5,
    "Tests of separability" = 6:7,
    "Specification test" = 8:8
  ),
  include.rsquared = F,
  include.adjrs = FALSE,
  include.nobs = F,
  include.rmse = FALSE,
  caption = "First stage"
)





# Second Stage Results
fakedata <- data.frame(matrix(rnorm(900), 100, 9))
fm <- lm(X1 ~ X2 + X3 + X4 + X5 + X6 + X7 + X8 + X9 - 1, data = fakedata)
stage <- "firststage"

cl <- c("Fama-French", "Industry & BE/ME", "Beta-sorted", "All portfolios", "FF3")

coef <- se <- pval <- list()
for (dt in c("famafrench", "industry", "beta", "all", "conditional")) {
  df <- results[[dt]][["secondstage"]]
  coef[[dt]] <- df$coef
  se[[dt]] <- df$se
  pval[[dt]] <-  2*(1-pnorm(abs(df$coef/df$se)))
  ## Add separability test
  coef[[dt]] <- c(coef[[dt]], df$sep, df$timesep)
  se[[dt]] <- c(se[[dt]], df$pv_sep, df$pv_timesep)
  ## Add J statistics
  coef[[dt]] <- c(coef[[dt]], df$J)
  se[[dt]] <- c(se[[dt]], pchisq(df$J, df = nrow(df$W)-5, lower = FALSE))
}


screenreg(
  list(fm, fm, fm, fm, fm), file = "secondstage.txt",
  stars = numeric(0),
  override.coef = coef,
  override.se = se,
  custom.coef.names = c(
    "$\\sigma$",
    "$\\gamma$",
    "$\\rho$",
    "$\\alpha$",
    "$\\beta$",
    "Test for $\\sigma = \\rho$",
    "Test for $\\sigma=1/\\gamma$",
    "$J-stat$"
  ),
  custom.model.names = cl,
  groups = list(
    "Parameters" = 1:5,
    "Tests of separability" = 6:7,
    "Specification test" = 8:8
  ),
  include.rsquared = F,
  include.adjrs = FALSE,
  include.nobs = F,
  include.rmse = FALSE
)


texreg(
  list(fm, fm, fm, fm, fm), file = "secondstage.tex",
  stars = numeric(0),
  override.coef = coef,
  override.se = se,
  custom.coef.names = c(
    "$\\sigma$",
    "$\\gamma$",
    "$\\rho$",
    "$\\alpha$",
    "$\\beta$",
    "Test for $\\sigma = \\rho$",
    "Test for $\\sigma=1/\\gamma$",
    "$J-stat$"
  ),
  custom.model.names = cl,
  groups = list(
    "Parameters" = 1:5,
    "Tests of separability" = 6:7,
    "Specification test" = 8:8
  ),
  include.rsquared = F,
  include.adjrs = FALSE,
  include.nobs = F,
  include.rmse = FALSE,
  caption = "First stage"
)
